<?php
namespace Teknisa\SM\Service;

use Teknisa\SM\Factory\Sm00002000ModeloPropostaEscopoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;

class Sm00002000ModeloPropostaEscopoService {
    
    protected $ModeloPropostaEscopoFactory;
    protected $Environment;
    protected $newCode;
    
    public function __construct (Sm00002000ModeloPropostaEscopoFactory $ModeloPropostaEscopoFactory, Environment $Environment, NewCode $newCode) {
        $this->ModeloPropostaEscopoFactory  = $ModeloPropostaEscopoFactory;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
    }
    
     /**
     * A função "validaEscopoProposta" verifica se o escopo 
     * que está sendo relacionado já está relacionado ao modelo.
     */
     
    public function validaEscopoProposta($tipoOrcamento, $orcaModelo){
    
        $paramsValidaEscopoProp = array(
            "NRTIPOORCAMENTO"   => implode(',', $tipoOrcamento),
            "NRORCAMENTOMODELO" => $orcaModelo,
            "NRORG"             => $this->Environment->getNrOrg()
        );
        
        $returnValidaEscopoProp = $this->ModeloPropostaEscopoFactory->validaEscopoProposta($paramsValidaEscopoProp);
        
        return $returnValidaEscopoProp[0]["VALIDAESCOPO"] != 0;
        
    }
    /**
     * Salva a edição dos modelos de documentos ligados a um escopo 
    **/
    public function editarDocumentosEscopoModelo($row, $nrorg, $cdoperador) {
        $situacoesRetiradas = false;
        $params = array(
            "NRRELACESCOPOMODELO" => $row["NRRELACESCOPOMODELO"],
            "NRORG"               => $nrorg
        );
        
        if(count($row["IDSITUACAORETIRADA"]) > 0) {
            $idSituacao = implode("','", $row["IDSITUACAORETIRADA"]);
            $this->ModeloPropostaEscopoFactory->excluirDocumentoSituacao($idSituacao, $params);
            $situacoesRetiradas = true;
        }
        
        if(count($row["IDSITUACAOADICIONADA"]) > 0 || ($situacoesRetiradas && !$row["IDSITUACAOORCAMENTO"])) {
            if(count($row["IDSITUACAOADICIONADA"]) == 0) {
                $this->ModeloPropostaEscopoFactory->deletarSituacaoNula($params);
                array_push($row["IDSITUACAOADICIONADA"], null);
            }
            
            foreach($row["IDSITUACAOADICIONADA"] as $idSituacao) {
                $nrorcaescopodoc = $this->newCode->getCodeByTableName("FCT_ORCAESCOPODOC", $nrorg, 1, 10, true);
                $params = array(
                    "NRORCAESCOPODOC"     => $nrorcaescopodoc,
                    "NRRELACESCOPOMODELO" => $row["NRRELACESCOPOMODELO"],
                    "NRSEQMODDOCFCT"      => $row["NRSEQMODDOCFCT"],
                    "IDPERMITEDOCX"       => $row["IDPERMITEDOCX"],
                    "IDATIVO"             => 'S',
                    "NRORG"               => $nrorg,
                    "IDSITUACAOORCAMENTO" => $idSituacao,
                    "CDOPERADOR"          => $cdoperador
                );
                
            }
            $this->ModeloPropostaEscopoFactory->inserirDocumentoSituacao($params);
        }
        
        $situacoesEditadas = array_diff($row["IDSITUACAOORCAMENTO"], $row["IDSITUACAOADICIONADA"]);
        if(count($situacoesEditadas) > 0) {
            $params = array(
                "NRSEQMODDOCFCT"      => $row["NRSEQMODDOCFCT"],
                "IDPERMITEDOCX"       => $row["IDPERMITEDOCX"],
                "NRRELACESCOPOMODELO" => $row["NRRELACESCOPOMODELO"],
                "CDOPERADOR"          => $cdoperador,
                "NRORG"               => $nrorg
            );
            $idSituacao = implode("','", $row["IDSITUACAOORCAMENTO"]);
            $this->ModeloPropostaEscopoFactory->editarDocumentoSituacao($idSituacao, $params);
        }
    }
    
    /**
     * Valida se o usuário está tentando inserir uma situação existente
    **/
    public function validarExistenciaIdsituacao($row, $nrorg){
        try {
            if($row["IDSITUACAOADICIONADA"]) {
                $idSituacao = implode("','", $row["IDSITUACAOADICIONADA"]);
                $params = array(
                    "NRRELACESCOPOMODELO" => $row["NRRELACESCOPOMODELO"],
                    "NRSEQMODDOCFCT"      => $row["NRSEQMODDOCFCT"],
                    "NRORG"               => $nrorg
                );
                
                $existe = $this->ModeloPropostaEscopoFactory->getModeloDocumentoSituacao($idSituacao, $params);
                if(count($existe) > 0 && $existe[0]["IDSITUACAOORCAMENTO"]) {
                    $situacoes = "SITUACAO_JA_CADASTRADA::" . $existe[0]["IDSITUACAOORCAMENTO"];
                    $situacoes = str_replace('ELABORACAO',  'Em Elaboração',     $situacoes);
                    $situacoes = str_replace('CONCLUIDO',   'Concluída',         $situacoes);
                    $situacoes = str_replace('ACEITO',      'Aceita',            $situacoes);
                    $situacoes = str_replace('CANCELADO',   'Cancelada',         $situacoes);
                    $situacoes = str_replace('ENVIADO',     'Documento Enviado', $situacoes);
                    throw new \Exception($situacoes);
                }
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    
    /**
     * insere os signatários padrões de modelo de proposta que foram buscados na parametrização SM 
    **/
    public function insereSignatariosPadroesOrca($row, $nrorg, $operador){
        try {
            
            $params = array(
                "NRORG" => $nrorg
            );
            
            $certsign = $this->ModeloPropostaEscopoFactory->validaTokenCertsign($params);
            
            if(count($certsign) > 0) {
                
                $params = array(
                    "NRORG" => $nrorg
                );
                
                $signatarios = $this->ModeloPropostaEscopoFactory->buscaSignatariosPadroes($params);
                
                if(count($signatarios) > 0) {
                
                  foreach($signatarios as $signatario){
                      
                       $params = array(
                           "NRORCAMENTOMODELO" => $row["NRORCAMENTOMODELO"],
                           "NRPARCNEGOCIO"     => $signatario["NRPARCNEGOCIO"],
                           "NRORG"             => $nrorg
                       );
                      
                       $valida = $this->ModeloPropostaEscopoFactory->validaSignatCadastrados($params);
                      
                       if ($valida == 0){
                      
                          $nrsignatpadrorca = $this->newCode->getCodeByTableName("FCT_SIGNATPADRMODORCA", $nrorg, 1, 10, true);
                          
                          $params= array(
                              "NRSIGNATPADRORCA"  => $nrsignatpadrorca,
                              "NRORCAMENTOMODELO" => $row["NRORCAMENTOMODELO"],
                              "NRPARCNEGOCIO"     => $signatario["NRPARCNEGOCIO"], 
                              "IDTIPOASSINATURA"  => $signatario["IDTIPOASSINATURA"], 
                              "IDTESTEMUNHA"      => $signatario["IDTESTEMUNHA"], 
                              "NRORDENACAO"       => $signatario["NRORDENACAO"], 
                              "NRORG"             => $nrorg,  
                              "OPERADOR"          => $operador
                          );
                          
                          $signatarios = $this->ModeloPropostaEscopoFactory->insereSignatariosPadroes($params);
                      }
                      
                   }  
                    
                }
                 
            }
            
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * A função "cadAutoSignatPadrOrca" realiza o cadastramento automático dos signatários padrões do modelo de proposta
     */
    public function cadAutoSignatPadrOrca($row){
        try {
            
            $nrorg = $this->Environment->getNrOrg();
            $operador = $this->Environment->getCdOperador();
            
            $params = array(
                "NRORG" => $nrorg
            );
            
            $certsign = $this->ModeloPropostaEscopoFactory->validaTokenCertsign($params);
            
            if(count($certsign) > 0) {
                
                $nrorcamentomodelo = $row["NRORCAMENTOMODELO"];
                $idtipoassinatura = $row["IDTIPOASSINATURA"]; 
                $idtestemunha = $row["IDTESTEMUNHA"];
                $nrordenacao = $row["NRORDENACAO"];
                $signatarios = $row["NRPARCNEGOCIO"];
                
                if (count($nrorcamentomodelo) > 0) {
                
                    foreach($nrorcamentomodelo as $modelo) {
                
                        if (count($signatarios) > 0) {
                        
                           foreach($signatarios as $signatario){
                              
                              $params = array(
                                  "NRORCAMENTOMODELO" => $modelo,
                                  "NRPARCNEGOCIO"     => $signatario,
                                  "NRORG"             => $nrorg
                              );
                              
                              $valida = $this->ModeloPropostaEscopoFactory->validaSignatCadastrados($params);
                              
                              if ($valida == 0){
                              
                                  $nrsignatpadrorca = $this->newCode->getCodeByTableName("FCT_SIGNATPADRMODORCA", $nrorg, 1, 10, true);
                                  
                                  $params= array(
                                      "NRSIGNATPADRORCA"  => $nrsignatpadrorca,
                                      "NRORCAMENTOMODELO" => $modelo,
                                      "NRPARCNEGOCIO"     => $signatario, 
                                      "IDTIPOASSINATURA"  => $idtipoassinatura, 
                                      "IDTESTEMUNHA"      => $idtestemunha, 
                                      "NRORDENACAO"       => $nrordenacao, 
                                      "NRORG"             => $nrorg,  
                                      "OPERADOR"          => $operador
                                  );
                                  
                                  $this->ModeloPropostaEscopoFactory->insereSignatariosPadroes($params);
                              }
                           }  
                        }
                    }
                }
            }
            
            $retorno = 'OK';
            
        } catch(\Exception $e) {
            $retorno = $e->getMessage();
            //throw $e;
        }
        
         return $retorno;
    }
    
    /**
     * A função "validaSignatPadrOrca" realiza a validação dos signatários padrões para evitar a inclusão repetida de um parceiro.
     */
    public function validaSignatPadrOrca($row){
        try {
            
           $nrorg = $this->Environment->getNrOrg();
           $nrparcnegocio = $row["NRPARCNEGOCIO"];
           $nrorcamentomodelo = $row["NRORCAMENTOMODELO"];
           
           $params = array(
              "NRORCAMENTOMODELO" => $nrorcamentomodelo,
              "NRPARCNEGOCIO"     => $nrparcnegocio,
              "NRORG"             => $nrorg
           );
          
           $valida = $this->ModeloPropostaEscopoFactory->validaSignatCadastrados($params);
        
           return $valida;    
        
        } catch(\Exception $e) {
            throw $e;
        }   
        
    }
    
    /**
     * Valida se o token da certsign está preenchido
     */
    public function validaIdCertsign($nrorg){
        try {
            
           $params = array(
                "NRORG" => $nrorg
           );
            
           $certsign = $this->ModeloPropostaEscopoFactory->validaTokenCertsign($params);
        
           $certsign = $certsign[0]["DSTOKENCERTSIGN"];   
           
           return $certsign;    
        
        } catch(\Exception $e) {
            throw $e;
        }   
    }
}
?>