<?php
namespace Teknisa\Model\Geral\Repositories;


class Operador extends \Doctrine\ORM\EntityRepository {
    public function getOperatorByNmAccess($nmAccess){
        $query = "SELECT CDOPERADOR, NMOPERADOR
                    FROM OPERADOR
                   WHERE UPPER(NMACESSO) = UPPER(:NMACESSO)";

        $params = array( 'NMACESSO' => $nmAccess );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return !empty($result) ? $result[0] : $result;
    }
    public static function verificaGrupoAlteraPonto($cdoperador){
        $sql = "SELECT COUNT(*) AS GRUPODPAP
                   FROM OPERADOR OP, OPERGRUPOP OG, AUTGRUPOP AG
                  WHERE OP.CDOPERADOR   = :P_CDOPERADOR
                    AND OP.CDOPERADOR   = OG.CDOPERADOR
                    AND OG.CDGRUPOPER   = AG.CDGRUPOPER
                    AND AG.IDAUTGRUPOP  = 'DPAP'";

        $param = array(
            'P_CDOPERADOR' => $cdoperador
        );

        $entityManager = \Setforms2\Shared\Registry::getEntityManager();
        $result = $entityManager->getConnection()->fetchAll($sql, $param);
        return count($result)>0 ? $result : array();
    }
    
    private function generateQueryPaginated($query, $startPos, $finalPos)
    {
        if ($startPos && $finalPos){
            $query = <<<SQL
          SELECT * 
            FROM (
                SELECT Q.*, rowNum rn
                 FROM ($query) Q
                WHERE rowNum <= $finalPos)
           WHERE rn >= $startPos
SQL;
        }
        return $query;
    }

    public function findAllOperador($cdCliente, $filter, $startPos=0, $finalPos=0)
    {
        $query  = <<<SQL
        SELECT CDOPERADOR, NMOPERADOR, IDATIVO, DSEMAILOPER, NMRAZSOCCLIE, CDCLIENTE, DECODE(CDCLIENTE, :P_CDCLIENTE, 'Y', 'N') AS IDEDITABLE
          FROM (SELECT OP.CDOPERADOR, OP.NMOPERADOR, OP.IDATIVO, OP.DSEMAILOPER, CL.NMRAZSOCCLIE, OP.CDCLIENTE
                  FROM OPERADOR OP
                  JOIN CLIENTE CL ON (CL.CDCLIENTE = OP.CDCLIENTE)
                 UNION
                SELECT OP.CDOPERADOR, OP.NMOPERADOR, OP.IDATIVO, OP.DSEMAILOPER, CL.NMRAZSOCCLIE, OP.CDCLIENTE
                  FROM OPERCLIE OC
                  JOIN OPERADOR OP ON (OP.CDOPERADOR = OC.CDOPERADOR)
                  JOIN CLIENTE CL ON (CL.CDCLIENTE = OP.CDCLIENTE)
                        )
                 WHERE 1 = 1
                 $filter
                 ORDER BY NMOPERADOR
SQL;

        $param = array(
            'P_CDCLIENTE' => $cdCliente
        );

        $query = $this->generateQueryPaginated($query, $startPos, $finalPos);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

    public function findAllOperadorResponsible($filter, $filterTeknisa, $finalPos = 0, $startPos = 0){
        $query  = <<<SQL
        SELECT CDOPERADOR, NMOPERADOR, IDATIVO, NMRAZSOCCLIE, CDCLIENTE
        FROM (SELECT OP.CDOPERADOR, OP.NMOPERADOR, OP.IDATIVO, CL.NMRAZSOCCLIE, OP.CDCLIENTE
                      FROM OPERADOR OP
                      JOIN CLIENTE CL ON (CL.CDCLIENTE = OP.CDCLIENTE)
                      AND OP.IDOPERATIVO = 'A'
                      AND CL.IDATIVO = 'S'
        WHERE 1=1
        $filterTeknisa)
        WHERE 1=1
        $filter
        ORDER BY NMOPERADOR

SQL;
        $query = $this->generateQueryPaginated($query, $startPos, $finalPos);
        return $this->getEntityManager()->getConnection()->fetchAll($query);
    }


    public function findAllOperadorRelationshipCliente($cdCliente, $userId, $filter, $startPos=0, $finalPos=0)
    {

        $query  = <<<SQL
        SELECT CDOPERADOR, NMOPERADOR, IDATIVO, DSEMAILOPER, NMRAZSOCCLIE, CDCLIENTE, DECODE(CDCLIENTE, :P_CDCLIENTE, 'Y', 'N') AS IDEDITABLE,NMACESSO
          FROM (SELECT OP.CDOPERADOR, OP.NMOPERADOR, OP.IDATIVO, OP.DSEMAILOPER, CL.NMRAZSOCCLIE, OP.CDCLIENTE,OP.NMACESSO
                  FROM OPERADOR OP
                  JOIN CLIENTE CL ON (CL.CDCLIENTE = OP.CDCLIENTE)
                 WHERE OP.CDCLIENTE = :P_CDCLIENTE
                 UNION
                SELECT OP.CDOPERADOR, OP.NMOPERADOR, OP.IDATIVO, OP.DSEMAILOPER, CL.NMRAZSOCCLIE, OP.CDCLIENTE,OP.NMACESSO
                  FROM OPERCLIE OC
                  JOIN OPERADOR OP ON (OP.CDOPERADOR = OC.CDOPERADOR)
                  JOIN CLIENTE CL ON (CL.CDCLIENTE = OP.CDCLIENTE)
                 WHERE OC.CDCLIENTE = :P_CDCLIENTE
                   AND EXISTS (SELECT O.CDOPERADOR
                                 FROM OPERADOR O
                                WHERE O.CDOPERADOR = :P_CDOPERADOR
                                  AND O.CDCLIENTE <> :P_CDCLIENTE))
                 WHERE 1 = 1
                 $filter
                 ORDER BY NMOPERADOR
SQL;

        $param = array(
            'P_CDCLIENTE' => $cdCliente,
            'P_CDOPERADOR' => $userId,
        );
        
        $query = $this->generateQueryPaginated($query, $startPos, $finalPos);


        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

     public function findAvailableCliente($userId, $cdCliente, $filter = ''){
          $sql = <<<SQL
          SELECT NMFANTCLIE,NMRAZSOCCLIE,CDCLIENTE,NRINSJURCLIE,CDCLIENTE AS CLIENTE
           FROM(
                  SELECT CLI.NMFANTCLIE,CLI.NMRAZSOCCLIE,CLI.CDCLIENTE,CLI.NRINSJURCLIE
                    FROM OPERCLIE OCL
                    JOIN OPERADOR OPE ON (OCL.CDOPERADOR  = OPE.CDOPERADOR AND OPE.CDOPERADOR = :CD_OPERADOR)
                    JOIN CLIENTE CLI ON (CLI.CDCLIENTE = OCL.CDCLIENTE AND CLI.IDATIVO = 'S')
                   WHERE OCL.CDCLIENTE <> :CD_CLIENTE
                   UNION
                   SELECT CLI.NMFANTCLIE,CLI.NMRAZSOCCLIE,CLI.CDCLIENTE,CLI.NRINSJURCLIE
                    FROM OPERADOR OPE
                    JOIN CLIENTE CLI ON (CLI.CDCLIENTE = OPE.CDCLIENTE AND CLI.IDATIVO = 'S')
                   WHERE OPE.CDOPERADOR = :CD_OPERADOR AND OPE.CDCLIENTE <> :CD_CLIENTE)
           WHERE 1 = 1
           $filter
           GROUP BY NMFANTCLIE,NMRAZSOCCLIE,CDCLIENTE,NRINSJURCLIE
           ORDER BY NMFANTCLIE ASC
SQL;

        $param = array(
            'CD_CLIENTE' => $cdCliente,
            'CD_OPERADOR' => $userId,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($sql, $param);
    }

    public function findIfCurrenctClienteStillAvaiable($userId, $cdCliente){
          $sql = <<<SQL
          SELECT NMFANTCLIE,NMRAZSOCCLIE,CDCLIENTE,NRINSJURCLIE,CDCLIENTE AS CLIENTE
           FROM(
                  SELECT CLI.NMFANTCLIE,CLI.NMRAZSOCCLIE,CLI.CDCLIENTE,CLI.NRINSJURCLIE
                    FROM OPERCLIE OCL
                    JOIN OPERADOR OPE ON (OCL.CDOPERADOR  = OPE.CDOPERADOR AND OPE.CDOPERADOR = :CD_OPERADOR)
                    JOIN CLIENTE CLI ON (CLI.CDCLIENTE = OCL.CDCLIENTE AND CLI.IDATIVO = 'S')
                   WHERE OCL.CDCLIENTE = :CD_CLIENTE
                   UNION
                   SELECT CLI.NMFANTCLIE,CLI.NMRAZSOCCLIE,CLI.CDCLIENTE,CLI.NRINSJURCLIE
                    FROM OPERADOR OPE
                    JOIN CLIENTE CLI ON (CLI.CDCLIENTE = OPE.CDCLIENTE AND CLI.IDATIVO = 'S')
                   WHERE OPE.CDOPERADOR = :CD_OPERADOR AND OPE.CDCLIENTE = :CD_CLIENTE)
           WHERE 1 = 1
           GROUP BY NMFANTCLIE,NMRAZSOCCLIE,CDCLIENTE,NRINSJURCLIE
           ORDER BY NMFANTCLIE ASC
SQL;

        $param = array(
            'CD_CLIENTE' => $cdCliente,
            'CD_OPERADOR' => $userId,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($sql, $param);
    }

    public function getPerfAcessParcByNrorgAndCdoperador($nrOrg, $cdoperador){
        $query = "SELECT PAC.NRPERFILACESSO AS ID, PAC.IDPERFILACESSOADM
                    FROM CP_PERFILACESSO PAC
                    JOIN CP_PERFACESSOPER PER ON (PER.NRPERFILACESSO = PAC.NRPERFILACESSO)
                   WHERE PAC.NRORG = :NRORG
                     AND PER.CDOPERADOR = :CDOPERADOR
                     AND PAC.IDATIVO = 'S'";

        $params = array( 'NRORG' => $nrOrg, 'CDOPERADOR' => $cdoperador );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }
    
    public function getPerfAcessParcByNrorgAndCdoperadorFull($nrOrg, $cdoperador){
        $query = "SELECT PAC.NRPERFILACESSO AS ID, PAC.IDPERFILACESSOADM
                    FROM CP_PERFILACESSO PAC
                    JOIN CP_PERFACESSOPER PER ON (PER.NRPERFILACESSO = PAC.NRPERFILACESSO)
                   WHERE PAC.NRORG = :NRORG
                     AND PER.CDOPERADOR = :CDOPERADOR
                     AND PAC.IDATIVO = 'S'
               UNION ALL
                  SELECT PP.NRPERFILACESSO AS ID, PA.IDPERFILACESSOADM
                    FROM PERFACESSPARC PP
                    JOIN PERFILACESSO PA ON PA.NRPERFILACESSO = PP.NRPERFILACESSO
                    JOIN OPERADOR OPE ON (OPE.NRPARCNEGOCIO = PP.NRPARCNEGOCIO)
                   WHERE PP.NRORG = :NRORG
                     AND OPE.CDOPERADOR = :CDOPERADOR
                     AND PP.IDATIVO = 'S'";

        $params = array( 'NRORG' => $nrOrg, 'CDOPERADOR' => $cdoperador );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    /**
     * @param $operador
     * @return array
     */
    public function getOperatorByNmAccessOrEmail($operador){
        $query = "SELECT CDOPERADOR AS ID, NMOPERADOR AS NAME, NRORG AS ORGANIZATION_ID, DSEMAILOPER AS EMAIL,
                         CDSENHAOPERWEB AS PASSWORD_MD5, CDSENHOPER AS PASSWORD_CRYPTO_PLUS,CDCLIENTE,
                         CASE WHEN IDOPERATIVO = 'A'
                              THEN 'Y'
                              ELSE 'N'
                          END AS IS_ACTIVE,
                         IDIDIOMA, NMACESSO, NMCOMPLOPER, CDEMPRESA, NRPARCNEGOCIO,
                         NRORG, NRORGTRAB, NRESTRUTTRAB, CDFILIALSELECIONADA
                    FROM OPERADOR
                   WHERE UPPER(NMACESSO) = UPPER(:OPERADOR)
                      OR DSEMAILOPER = :OPERADOR";

        $params = array( 'OPERADOR' => $operador );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return !empty($result) ? $result[0] : $result;
    }

    public function currentCliente($cdCliente){
        $sql = <<<SQL
               SELECT CLI.NMFANTCLIE,CLI.NMRAZSOCCLIE,CLI.CDCLIENTE,CLI.NRINSJURCLIE,OMG.IDALLOWPRODUCTCALL
                FROM CLIENTE CLI
                JOIN ORGANIZACAO ORG ON(ORG.NRPARCNEGOCIO = CLI.NRPARCNEGOCIO)
           LEFT JOIN OM_PARAM_ORG OMG ON (OMG.NRORG = ORG.NRORG)
               WHERE CLI.CDCLIENTE = :CD_CLIENTE
SQL;

        $param = array(
            'CD_CLIENTE' => $cdCliente,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($sql, $param);
    }

    public function findProfileForAddProfile($nrparcnegocio){
          $sql = <<<SQL
                 SELECT PAC.NRPERFILACESSO,PAC.NMPERFILACESSO
              FROM PERFILACESSO PAC
         LEFT JOIN PERFACESSPARC PER ON (PER.NRPERFILACESSO = PAC.NRPERFILACESSO)
             WHERE PAC.IDATIVO = 'S' AND
             PAC.NRPERFILACESSO NOT IN(
                       SELECT PAC.NRPERFILACESSO
                        FROM PERFILACESSO PAC
                   LEFT JOIN PERFACESSPARC PER ON (PER.NRPERFILACESSO = PAC.NRPERFILACESSO)
                       WHERE (PER.NRPARCNEGOCIO = :NRPARCNEGOCIO)
                    GROUP BY PAC.NRPERFILACESSO,PAC.NMPERFILACESSO
                   )
          GROUP BY PAC.NRPERFILACESSO,PAC.NMPERFILACESSO
          ORDER BY PAC.NMPERFILACESSO
SQL;

        $param = array(
            'NRPARCNEGOCIO' => $nrparcnegocio
        );
        return $this->getEntityManager()->getConnection()->fetchAll($sql, $param);
    }

    public function setEntityManager($em) {
        $this->_em = $em;
    }

    public function updateOrgOperator($dbUser, $email, $nmAccess, $name, $orgId, $code, $entityManager = null) {
        $query = "
        UPDATE OPERADOR
           SET DSEMAILOPER = :DSEMAILOPER,
               NMACESSO    = :NMACESSO,
               NMOPERADOR  = :NMOPERADOR
         WHERE NRORG      = :NRORG
           AND CDOPERADOR = :CDOPERADOR
        ";
        $param = array(
            "DSEMAILOPER" => $email,
            "NMACESSO" => $nmAccess,
            "NMOPERADOR" => $name,
            "NRORG" => $orgId,
            "CDOPERADOR" => $code
        );
        $entityManager = $entityManager ? $entityManager : $this->getEntityManager();
        $entityManager->getConnection()->executeQuery($query, $param);
    }

    public function findOrgOperator($orgOperatorEmail, $orgId, $entityManager = null){
        $sql = <<<SQL
                 SELECT CDOPERADOR, NMOPERADOR, NMACESSO, DSEMAILOPER
                   FROM OPERADOR
                  WHERE NRORG = :NRORG
                    AND DSEMAILOPER = :EMAIL
SQL;

        $param = array( "NRORG" => $orgId, "EMAIL" => $orgOperatorEmail );
        $entityManager = $entityManager ? $entityManager : $this->getEntityManager();
        return $entityManager->getConnection()->fetchAll($sql, $param);
    }
    
    public function findProfileForAddProfileCP($cdCliente,$cdoperador){
        $sql = <<<SQL
               SELECT PAC.NRPERFILACESSO,PAC.NMPERFILACESSO
              FROM CP_PERFILACESSO PAC
         LEFT JOIN CP_PERFACESSOPER PER ON (PER.NRPERFILACESSO = PAC.NRPERFILACESSO)
             WHERE PAC.IDATIVO = 'S' AND
             PAC.NRPERFILACESSO NOT IN(
                       SELECT PAC.NRPERFILACESSO
                        FROM CP_PERFILACESSO PAC
                   LEFT JOIN CP_PERFACESSOPER PER ON (PER.NRPERFILACESSO = PAC.NRPERFILACESSO)
                       WHERE (PER.CDOPERADOR = :CDOPERADOR AND PAC.CDCLIENTE = :CDCLIENTE)
                    GROUP BY PAC.NRPERFILACESSO,PAC.NMPERFILACESSO
                   )
          GROUP BY PAC.NRPERFILACESSO,PAC.NMPERFILACESSO
          ORDER BY PAC.NMPERFILACESSO
SQL;

        $param = array(
            'CDOPERADOR' => $cdoperador,
            'CDCLIENTE' => $cdCliente,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($sql, $param);
    }
}